package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesShiftService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesShift;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesShiftService extends BaseMesService<MesShift> implements IMesShiftService {

    protected void setPackQueryBean(MesShift bean, DdlPackBean packBean) {

        DdlPreparedPack.getNumEqualPack(bean.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShiftCode(), "shiftCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getShiftName(), "shiftName", packBean);

    }

    protected void onInsertBean(MesShift item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getAreaCode(), "区域不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次代码不能为空");
        ValidatorBean.checkNotNull(item.getShiftName(), "班次名称不能为空");
        ValidatorBean.checkNotNull(item.getStartTime(), "开始时间不能为空");
        ValidatorBean.checkNotNull(item.getEndTime(), "结束时间不能为空");
        ValidatorBean.checkNotNull(item.getBeginDate(), "执行开始日期不能为空");
        ValidatorBean.checkNotNull(item.getEndDate(), "执行结束日期不能为空");
/*
        ValidatorBean.checkNotNull(item.getWorkerQty(), "作业人数不能为空");
*/

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getAreaCode(), "areaCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", cellBinPackBean);
        cellBinPackBean.setWhereAppend(cellBinPackBean.getWhereAppend()+" " +
                " and ((model.beginDate <= '"+item.getBeginDate()+"' and model.endDate >= '"+item.getBeginDate()+"')"+
                " or (model.beginDate <= '"+item.getEndDate()+"' and model.endDate >= '"+item.getEndDate()+"')" +
                " or (model.beginDate >= '"+item.getBeginDate()+"' and model.endDate <= '"+item.getEndDate()+"'))" );
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】区域【%s】产线【%s】班次代码生效时间内已经存在有效数据，请检查数据", item.getAreaCode(), item.getWorkCenterCode(), item.getShiftCode())
                    .build();
        }
    }


    protected void onUpdateBean(MesShift item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getAreaCode(), "区域不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次代码不能为空");
        ValidatorBean.checkNotNull(item.getShiftName(), "班次名称不能为空");
        ValidatorBean.checkNotNull(item.getStartTime(), "开始时间不能为空");
        ValidatorBean.checkNotNull(item.getEndTime(), "结束时间不能为空");
        ValidatorBean.checkNotNull(item.getBeginDate(), "执行开始日期不能为空");
        ValidatorBean.checkNotNull(item.getEndDate(), "执行结束日期不能为空");
/*
        ValidatorBean.checkNotNull(item.getWorkerQty(), "作业人数不能为空");
*/

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getAreaCode(), "areaCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", cellBinPackBean);
        cellBinPackBean.setWhereAppend(cellBinPackBean.getWhereAppend()+" " +
                " and ((model.beginDate <= '"+item.getBeginDate()+"' and model.endDate >= '"+item.getBeginDate()+"')"+
                " or (model.beginDate <= '"+item.getEndDate()+"' and model.endDate >= '"+item.getEndDate()+"')" +
                " or (model.beginDate >= '"+item.getBeginDate()+"' and model.endDate <= '"+item.getEndDate()+"'))" );
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】区域【%s】产线【%s】班次代码生效时间内已经存在有效数据，请检查数据", item.getAreaCode(), item.getWorkCenterCode(), item.getShiftCode())
                    .build();
        }
    }

    @Override
    public List<MesShift> findMesShift(MesShift mesShift) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesShift.getOrganizeCode());
        String nowDate = TimeTool.getNowTime(true);
        DdlPreparedPack.getNumEqualPack(mesShift.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(mesShift.getShiftCode(), "shiftCode", packBean);
        DdlPreparedPack.getStringEqualPack(mesShift.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(mesShift.getShiftName(), "shiftName", packBean);
        DdlPreparedPack.getStringSmallerPack(nowDate,"beginDate",packBean);
        DdlPreparedPack.getStringBiggerPack(nowDate,"endDate",packBean);
        return baseRDao.findByHqlWhere(packBean);
    }
}
